# HTTP Status Codes
OK = 200
CREATED = 201
ACCEPTED = 202
NO_CONTENT = 204

BAD_REQUEST = 400
UNAUTHORIZED = 401
FORBIDDEN = 403
NOT_FOUND = 404
METHOD_NOT_ALLOWED = 405
NOT_ACCEPTABLE = 406
CONFLICT = 409
GONE = 410
PRECONDITION_FAILED = 412
UNSUPPORTED_MEDIA_TYPE = 415
EXPECTATION_FAILED = 417
I_AM_A_TEAPOT = 418
UNPROCESSABLE_ENTITY = 422
LOCKED = 423
FAILED_DEPENDENCY = 424
TOO_MANY_REQUESTS = 429
UNAVAILABLE_FOR_LEGAL_REASONS = 451

APPLICATION_ERROR = 500
METHOD_NOT_IMPLEMENTED = 501
UNAVAILABLE = 503
